@@ -15,7 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program (see the file COPYING); if not, write to the
- * Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * Free Software Foundation, Inc.,
+ * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
  *
  ****************************************************************
  * $Id: extern.h,v 1.18 1994/05/31 12:31:57 mlschroe Exp $ FAU
@@ -23,7 +24,7 @@
 
 
 /* screen.c */
-extern void  main __P((int, char **));
+extern int   main __P((int, char **));
 extern sigret_t SigHup __P(SIGPROTOARG);
 extern void  eexit __P((int));
 extern void  Detach __P((int));
@@ -46,6 +47,7 @@ extern void  Panic __P(());
 extern void  DisplaySleep __P((int));
 extern void  Finit __P((int));
 extern void  MakeNewEnv __P((void));
+extern char *MakeWinMsg __P((char *, struct win *, int));
 
 /* ansi.c */
 extern void  Activate __P((int));
@@ -54,11 +56,9 @@ extern void  ResetCharsets __P((struct win *));
 extern void  WriteString __P((struct win *, char *, int));
 extern void  NewAutoFlow __P((struct win *, int));
 extern void  Redisplay __P((int));
-extern void  CheckLP __P((int));
-extern void  MakeBlankLine __P((char *, int));
 extern void  SetCurr __P((struct win *));
 extern void  ChangeAKA __P((struct win *, char *, int));
-extern void  AddLineToHist __P((struct win *, char **, char **, char **));
+extern void  SetCharsets __P((struct win *, char *));
 
 /* fileio.c */
 extern void  StartRc __P((char *));
@@ -69,7 +69,7 @@ extern int   secopen __P((char *, int, int));
 extern void  WriteFile __P((int));
 extern char *ReadFile __P((char *, int *));
 extern void  KillBuffers __P((void));
-extern char *expand_vars __P((char *));
+extern char *expand_vars __P((char *, struct display *));
 
 /* tty.c */
 extern int   OpenTTY __P((char *));
@@ -99,10 +99,11 @@ extern void  ISearch __P((int));
 
 /* input.c */
 extern void  inp_setprompt __P((char *, char *));
-extern void  Input __P((char *, int, void (*)(), int));
+extern void  Input __P((char *, int, int, void (*)(), char *));
+extern int   InInput __P((void));
 
 /* help.c */
-extern void  exit_with_usage __P((char *));
+extern void  exit_with_usage __P((char *, char *, char *));
 extern void  display_help __P((void));
 extern void  display_copyright __P((void));
 extern void  display_displays __P((void));
@@ -146,6 +147,9 @@ extern int   OpenPTY __P((char **));
 /* process.c */
 extern void  InitKeytab __P((void));
 extern void  ProcessInput __P((char *, int));
+#ifdef MAPKEYS
+extern void  ProcessInput2 __P((char *, int));
+#endif
 extern int   FindCommnr __P((char *));
 extern void  DoCommand __P((char **));
 extern void  KillWindow __P((struct win *));
@@ -158,7 +162,7 @@ extern int   IsNumColon __P((char *, int, char *, int));
 extern void  ShowWindows __P((void));
 extern int   WindowByNoN __P((char *));
 #ifdef COPY_PASTE
-extern int   CompileKeys __P((char *, char *));
+extern int   CompileKeys __P((char *, unsigned char *));
 #endif
 
 /* termcap.c */
@@ -167,7 +171,10 @@ extern char *MakeTermcap __P((int));
 extern char *gettermcapstring __P((char *));
 #ifdef MAPKEYS
 extern int   remap __P((int, int));
+extern void  CheckEscape __P((void));
 #endif
+extern int   CreateTransTable __P((char *));
+extern void  FreeTransTable __P((void));
 
 /* attacher.c */
 extern int   Attach __P((int));
@@ -196,21 +203,25 @@ extern void  ClearDisplay __P((void));
 extern void  Clear __P((int, int, int, int, int, int, int));
 extern void  RefreshLine __P((int, int, int, int));
 extern void  RefreshStatus __P((void));
-extern void  DisplayLine __P((char *, char *, char *, char *, char *, char *, int, int, int));
+extern void  DisplayLine __P((struct mline *, struct mline *, int, int, int));
+
+extern void  CDisplayLine __P((struct mline *, int, int, int, int, int));
 extern void  FixLP __P((int, int));
 extern void  GotoPos __P((int, int));
 extern int   CalcCost __P((char *));
-extern void  ScrollH __P((int, int, int, int, char *, char *, char *));
+extern void  ScrollH __P((int, int, int, int, struct mline *));
 extern void  ScrollV __P((int, int, int, int, int));
 extern void  ChangeScrollRegion __P((int, int));
 extern void  InsertMode __P((int));
 extern void  KeypadMode __P((int));
 extern void  CursorkeysMode __P((int));
 extern void  ReverseVideo __P((int));
-extern void  CursorInvisible __P((int));
+extern void  CursorVisibility __P((int));
 extern void  SetFont __P((int));
 extern void  SetAttr __P((int));
-extern void  SetAttrFont __P((int, int));
+extern void  SetColor __P((int));
+extern void  SetRendition __P((struct mchar *));
+extern void  SetRenditionMline __P((struct mline *, int));
 extern void  MakeStatus __P((char *));
 extern void  RemoveStatus __P((void));
 extern void  SetLastPos __P((int, int));
@@ -230,15 +241,14 @@ extern int   badkanji __P((char *, int));
 #endif
 
 /* resize.c */
-extern int   ChangeScrollback __P((struct win *, int, int));
-extern int   ChangeWindowSize __P((struct win *, int, int));
+extern int   ChangeWindowSize __P((struct win *, int, int, int));
 extern void  ChangeScreenSize __P((int, int, int));
 extern void  CheckScreenSize __P((int));
 extern void  DoResize __P((int, int));
 extern char *xrealloc __P((char *, int));
 
 /* socket.c */
-extern int   FindSocket __P((int *, int *, char *));
+extern int   FindSocket __P((int *, int *, int *, char *));
 extern int   MakeClientSocket __P((int));
 extern int   MakeServerSocket __P((void));
 extern int   RecoverSocket __P((void));
@@ -283,6 +293,14 @@ extern void  xsetegid  __P((int));
 #endif
 extern int   AddXChar __P((char *, int));
 extern int   AddXChars __P((char *, int, char *));
+extern void  opendebug __P((int, int));
+#ifdef USEVARARGS
+# ifndef HAVE_VSNPRINTF
+extern int   xvsnprintf __P((char *, int, char *, va_list));
+# endif
+#else
+extern int   xsnprintf __P(());
+#endif
 
 /* acl.c */
 #ifdef MULTIUSER
@@ -297,9 +315,10 @@ extern int   AclUserAddGroup __P((char *, char *));
 extern int   AclUserDelGroup __P((char *, char *));
 extern int   AclCheckPermWin __P((struct user *, int, struct win *));
 extern int   AclCheckPermCmd __P((struct user *, int, struct comm *));
-extern int   AclSetPerm __P((struct user *, char *, char *));
+extern int   AclSetPerm __P((struct user *, struct user *, char *, char *));
+extern int   UsersAcl __P((struct user *, int, char **));
 extern void  AclWinSwap __P((int, int));
-extern int   NewWindowAcl __P((struct win *));
+extern int   NewWindowAcl __P((struct win *, struct user *));
 #endif /* MULTIUSER */
 extern struct user **FindUserPtr __P((char *));
 extern int   UserAdd __P((char *, char *, struct user **));
